Package: rcatfish
Version: 1.0.2
Date: 2026-2-18
Title: An R Interface to the California Academy of Sciences Eschmeyer's
        Catalog of Fishes
Authors@R: 
  c(person(given = "Samuel R.",
           family = "Borstein",
           role = c("aut", "cre"),
           email = "sam@borstein.com"),
    person(given = "Brandon",
           family = "Dominy",
           role = "aut",
		   email = "bed47@txstate.edu"),
    person(given = "Brian",
           family = "O'Meara",
           role = "aut",
		   email = "bomeara@utk.edu"))
Depends: R (>= 4.4.0)
Imports: curl (>= 5.2.1), httr (>= 1.4.7), jsonlite (>= 1.8.9),
        lubridate (>= 1.9.3), magrittr (>= 2.0.3), qdapRegex (>=
        0.7.2), RCurl (>= 1.95-4.11), rfishbase (>= 5.0.0), rvest (>=
        0.3.2), stringr (>= 1.3.1), tidyr (>= 0.8.1), xml2 (>= 1.2.0)
Description: Accesses the California Academy of Sciences Eschmeyer's Catalog of Fishes in R using web requests. The Catalog of fishes is the leading authority in fish taxonomy. Functions in the package allow users to search for fish taxa and valid names, retrieve taxonomic references, retrieve monthly taxonomic changes, obtain natural history collection information, and see the number of species by taxonomic group. For more information on the Catalog: Fricke, R., Eschmeyer, W. N. & R. van der Laan (eds) 2025. ESCHMEYER'S CATALOG OF FISHES <https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp>.
License: GPL (>= 2)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-02-18 15:25:21 UTC; Sam
Author: Samuel R. Borstein [aut, cre],
  Brandon Dominy [aut],
  Brian O'Meara [aut]
Maintainer: Samuel R. Borstein <sam@borstein.com>
Repository: CRAN
Date/Publication: 2026-02-19 09:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:50:10 UTC; unix
