# Generated by roxygen2: do not edit by hand

export(rcatfish_classification)
export(rcatfish_collections)
export(rcatfish_glossary)
export(rcatfish_journals)
export(rcatfish_references)
export(rcatfish_search)
export(rcatfish_species_by)
export(rcatfish_updates)
export(rcatfish_version)
import(rvest)
import(stringr)
importFrom(magrittr,"%>%")
importFrom(xml2,read_html)
